#!/usr/bin/python
# -*- coding: utf-8 -*-
import unittest, os

from indicoio import config
from indicoio import image_recognition

DIR = os.path.dirname(os.path.realpath(__file__))
class ImageRecognitionTest(unittest.TestCase):
    def setUp(self):
        self.api_key = config.api_key

    def test_single_image_recognition(self):
        test_data = os.path.normpath(os.path.join(DIR, "data", "fear.png"))
        response = image_recognition(test_data, api_key = self.api_key, top_n=3)
        self.assertIsInstance(response, dict)
        self.assertEqual(len(response), 3)
        self.assertIsInstance(list(response.values())[0], float)

    def test_batch_image_recognition(self):
        test_data = os.path.normpath(os.path.join(DIR, "data", "fear.png"))
        response = image_recognition([test_data, test_data], api_key = self.api_key, top_n=3)
        self.assertIsInstance(response, list)
        self.assertIsInstance(response[0], dict)
        self.assertEqual(len(response[0]), 3)
        self.assertIsInstance(list(response[0].values())[0], float)

    def test_expected_response(self):
        test_data = os.path.normpath(os.path.join(DIR, "data", "keyboard.jpg"))
        response = image_recognition(test_data, api_key = self.api_key, top_n=3)
        assert "space bar" in response.keys()

if __name__ == "__main__":
    unittest.main()



import unittest

from indicoio import config
from indicoio import keywords


class KeywordsV2(unittest.TestCase):
    def setUp(self):
        self.api_key = config.api_key
        config.url_protocol = "http"

        if not all(self.api_key):
            raise SkipTest

    def tearDown(self):
        config.url_protocol = "https"

    def test_keywords_v2(self):
        test_data = "A working api is key to the success of our young company"
        response = keywords(test_data, version=2)
        self.assertTrue(isinstance(response, dict))
        self.assertTrue(all([key in test_data for key in response.keys()]))

    def test_batch_keywords_v2(self):
        test_data = ["A working api is key to the success of our young company"]
        response = keywords(test_data, version=2)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(all([key in test_data[0] for key in response[0].keys()]))



#!/usr/bin/python
# -*- coding: utf-8 -*-
import unittest, os

from indicoio import config
from indicoio import sentiment
from indicoio import image_features

DIR = os.path.dirname(os.path.realpath(__file__))
class TestVersioning(unittest.TestCase):
    def setUp(self):
        self.api_key = config.api_key

    def test_specify_version(self):
        test_data = ['Worst song ever', 'Best song ever']
        response = sentiment(test_data, version="1")
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 2)
        self.assertTrue(response[0] < .5)
        self.assertTrue(response[1] > .5)

    def test_image_features_v2(self):
        test_data = os.path.normpath(os.path.join(DIR, "data", "fear.png"))
        response = image_features(test_data, version="2")
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 4096)

if __name__ == "__main__":
    unittest.main()



import unittest
from indicoio.utils.errors import IndicoError
from indicoio.custom import Collection, collections
import time

collection_name = "__test_python_text__"
test_data = [['input 1', 'label 1'], ['input 2', 'label 2'], ['input 3', 'label 3'], ['input 4', 'label 4']]

image_collection_name = "__test_python_image__"
image_test_data = [
    ["https://i.imgur.com/xUX1rvY.png", "dog"],
    ["https://i.imgur.com/xUX1rvY.png", "dog"],
    ['https://i.imgur.com/2Q0EWRz.jpg', 'cat'],
    ['https://i.imgur.com/XhUDCMP.jpg', 'cat']
]


class CustomAPIsTestCase(unittest.TestCase):

    def tearDown(self):
        try:
            Collection(collection_name).clear()
        except IndicoError:
            pass

        try:
            Collection(image_collection_name).clear()
        except IndicoError:
            pass

    def test_add_predict(self):
        collection = Collection(collection_name)
        collection.add_data(test_data)
        collection.train()
        collection.wait()
        result = collection.predict(test_data[0][0])
        assert test_data[0][1] in result.keys()

    def test_list_collection(self):
        collection = Collection(collection_name)
        collection.add_data(test_data)
        collection.train()
        collection.wait()
        assert collections()[collection_name]

    def test_add_large_batch(self):
        collection = Collection(collection_name)
        collection.add_data(test_data*100)
        collection.train()
        collection.wait()
        result = collection.predict(test_data[0][0])
        assert test_data[0][1] in result.keys()

    def test_add_image_batch(self):
        collection = Collection(image_collection_name)
        collection.add_data(image_test_data)
        collection.train()
        collection.wait()
        result = collection.predict(image_test_data[0][0])
        assert image_test_data[0][1] in result.keys()

    def test_clear_example(self):
        collection = Collection(collection_name)
        collection.add_data(test_data)
        collection.train()
        collection.wait()
        result = collection.predict(test_data[0][0])
        assert test_data[0][1] in result.keys()
        collection.remove_example(test_data[0][0])
        collection.train()
        collection.wait()
        result = collection.predict(test_data[0][0])
        assert test_data[0][1] not in result.keys()

    def test_clear_collection(self):
        collection = Collection(collection_name)
        collection.add_data(test_data)
        collection.train()
        collection.wait()
        assert collections()[collection_name]
        collection.clear()
        assert not collections().get(collection_name)



from indicoio.utils.api import api_handler
from indicoio.utils.decorators import detect_batch_decorator


@detect_batch_decorator
def named_entities(text, cloud=None, batch=False, api_key=None, version=None, **kwargs):
    """
    Given input text, returns named entities (proper nouns) found in the text

    Example usage:

    .. code-block:: python

       >>> text = "London Underground's boss Mike Brown warned that the strike ..."
       >>> entities = indicoio.named_entities(text)
       {u'London Underground': {u'categories': {u'location': 0.583755654607989,
          u'organization': 0.07460487821791033,
          u'person': 0.07304850776658672,
          u'unknown': 0.2685909594075139},
         u'confidence': 0.846188063604044},
        u'Mike Brown': {u'categories': {u'location': 0.025813884950623898,
          u'organization': 0.06661470013014613,
          u'person': 0.08723850624560824,
          u'unknown': 0.8203329086736217},
         u'confidence': 0.8951793008234012}}

    :param text: The text to be analyzed.
    :type text: str or unicode
    :rtype: Dictionary of named entity probability pairs
    """
    url_params = {"batch": batch, "api_key": api_key, "version": version}
    return api_handler(text, cloud=cloud, api="namedentities", url_params=url_params, **kwargs)


@detect_batch_decorator
def people(text, cloud=None, batch=None, api_key=None, version=None, **kwargs):
    """
    Given input text, returns references to specific persons found in the text

    Example usage:

    .. code-block:: python

       >>> text = "London Underground's boss Mike Brown warned that the strike ..."
       >>> entities = indicoio.people(text)
        [
          {
            u'text': "Mike Brown",
            u'confidence': 0.09470917284488678,
            u'position': [26, 36]
          },
          ...
        ]

    :param text: The text to be analyzed.
    :type text: str or unicode
    :rtype: Dictionary of language probability pairs
    """
    url_params = {"batch": batch, "api_key": api_key, "version": version}
    return api_handler(text, cloud=cloud, api="people", url_params=url_params, **kwargs)


@detect_batch_decorator
def places(text, cloud=None, batch=None, api_key=None, version=None, **kwargs):
    """
    Given input text, returns references to specific places found in the text

    Example usage:

    .. code-block:: python

       >>> text = "London Underground's boss Mike Brown warned that the strike ..."
       >>> entities = indicoio.places(text)
        [
          {
            u'text': "London",
            u'confidence': 0.18549786508083344,
            u'position': [0, 6]
          },
          ...
        ]

    :param text: The text to be analyzed.
    :type text: str or unicode
    :rtype: Dictionary of language probability pairs
    """
    url_params = {"batch": batch, "api_key": api_key, "version": version}
    return api_handler(text, cloud=cloud, api="places", url_params=url_params, **kwargs)



@detect_batch_decorator
def organizations(text, cloud=None, batch=None, api_key=None, version=None, **kwargs):
    """
    Given input text, returns references to specific organizations found in the text

    Example usage:

    .. code-block:: python

       >>> text = "London Underground's boss Mike Brown warned that the strike ..."
       >>> entities = indicoio.organizations(text)
        [
          {
            u'text': "London Underground",
            u'confidence': 0.8643872141838074,
            u'position': [0, 18]
          }
        ]

    :param text: The text to be analyzed.
    :type text: str or unicode
    :rtype: Dictionary of language probability pairs
    """
    url_params = {"batch": batch, "api_key": api_key, "version": version}
    return api_handler(text, cloud=cloud, api="organizations", url_params=url_params, **kwargs)



import time

from indicoio.utils.api import api_handler
from indicoio.utils.decorators import detect_batch
from indicoio.utils.image import image_preprocess
from indicoio.utils.errors import IndicoError


class Collection(object):

    def __init__(self, collection, *args, **kwargs):
        self.collection = collection
        self.domain = kwargs.get("domain")

    def add_data(self, data, cloud=None, batch=False, api_key=None, version=None, **kwargs):
        """
        This is the basic training endpoint. Given a piece of text and a score, either categorical
        or numeric, this endpoint will train a new model given the additional piece of information.

        Inputs
        data - List: The text and collection/score associated with it. The length of the text (string) should ideally
          be longer than 100 characters and contain at least 10 words. While the API will support
          shorter text, you will find that the accuracy of results improves significantly with longer
          examples. For an additional fee, this end point will support image input as well. The collection/score
          can be a string or float. This is the variable associated with the text. This can either be categorical
          (the tag associated with the post) or numeric (the number of Facebook shares the post
          received). However it can only be one or another within a given label.
        collection (optional) - String: This is an identifier for the particular model being trained. The indico
          API allows you to train a number of different models. If the collection is not provided, indico
          will add a default label.
        domain (optional) - String: This is an identifier that helps determine the appropriate techniques for indico
          to use behind the scenes to train your model.  One of {"standard", "topics"}.
        api_key (optional) - String: Your API key, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.
        cloud (optional) - String: Your private cloud domain, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.

        Example usage:

        .. code-block:: python

           >>> text = "London Underground's boss Mike Brown warned that the strike ..."
           >>> indicoio.add_data([[text, .5]])
        """
        batch = isinstance(data[0], list)
        if batch:
            X, Y = zip(*data)
            X = image_preprocess(X, batch=batch)
            data = map(list, zip(X, Y))
        else:
            data[0] = image_preprocess(data[0], batch=batch)

        kwargs['collection'] = self.collection
        if self.domain:
            kwargs["domain"] = self.domain
        url_params = {"batch": batch, "api_key": api_key, "version": version, 'method': "add_data"}
        return api_handler(data, cloud=cloud, api="custom", url_params=url_params, **kwargs)


    def train(self, cloud=None, batch=False, api_key=None, version=None, **kwargs):
        """
        This is the basic training endpoint. Given an existing dataset this endpoint will train a model.

        Inputs
        collection - String: the name of the collection to train a model using
        api_key (optional) - String: Your API key, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.
        cloud (optional) - String: Your private cloud domain, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.

        Example usage:

        .. code-block:: python

           >>> indicoio.train(collection)
        """
        kwargs['collection'] = self.collection
        url_params = {"batch": batch, "api_key": api_key, "version": version, 'method': "train"}
        return api_handler(self.collection, cloud=cloud, api="custom", url_params=url_params, private=True, **kwargs)


    def predict(self, data, cloud=None, batch=False, api_key=None, version=None, **kwargs):
        """
        This is the prediction endpoint. This will be the primary interaction point for all predictive
        analysis.

        Inputs
        data - String: The text example being provided to the API. As a general rule, the data should be as
          similar to the examples given to the train function (above) as possible. Because language
          in different domains is used very differently the accuracy will generally drop as the
          difference between this text and the training text increases. Base64 encoded image data, image urls, and
          text content are all valid.
        domain (optional) - String: This is an identifier that helps determine the appropriate techniques for indico
          to use behind the scenes to train your model.  One of {"standard", "topics"}.
        collection (optional) - String: This is an identifier for the particular model to use for prediction. The
          response format for the given label will match the format of the training examples
        api_key (optional) - String: Your API key, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.
        cloud (optional) - String: Your private cloud domain, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.

        Example usage:

        .. code-block:: python

          >>> text = "I am Sam. Sam I am."
          >>> prediction = indicoio.predict(text)
          .75
        """
        batch = detect_batch(data)
        if self.domain:
            kwargs["domain"] = self.domain
        kwargs['collection'] = self.collection
        data = image_preprocess(data, batch=batch)
        url_params = {"batch": batch, "api_key": api_key, "version": version}
        return api_handler(data, cloud=cloud, api="custom", url_params=url_params, private=True, **kwargs)


    def clear(self, cloud=None, api_key=None, version=None, **kwargs):
        """
        This is an API made to remove all of the data associated from a given colletion. If there's been a data
        corruption issue, or a large amount of incorrect data has been fed into the API it is often difficult
        to correct. This allows you to clear a colletion and start from scratch. Use with caution! This is not
        reversible.

        Inputs
        colletion - String: the colletion from which you wish to remove the specified text.
        api_key (optional) - String: Your API key, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.
        cloud (optional) - String: Your private cloud domain, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.

        Example usage:

        .. code-block:: python

          >>> indicoio.clear_collection("popularity_predictor")

        """
        kwargs['collection'] = self.collection
        url_params = {"batch": False, "api_key": api_key, "version": version, "method": "clear_collection"}
        return api_handler(None, cloud=cloud, api="custom", url_params=url_params, private=True, **kwargs)

    def remove_example(self, data, cloud=None, batch=False, api_key=None, version=None, **kwargs):
        """
        This is an API made to remove a single instance of training data. This is useful in cases where a
        single instance of content has been modified, but the remaining examples remain valid. For
        example, if a piece of content has been retagged.

        Inputs
        data - String: The exact text you wish to remove from the given collection. If the string
          provided does not match a known piece of text then this will fail. Again, this is required if
          an id is not provided, and vice-versa.
        collection - String: the collection from which you wish to remove the specified text.
        api_key (optional) - String: Your API key, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.
        cloud (optional) - String: Your private cloud domain, required only if the key has not been declared
          elsewhere. This allows the API to recognize a request as yours and automatically route it
          to the appropriate destination.

        Example usage:

        .. code-block:: python

          >>> indicoio.remove_example(text="I am Sam. Sam I am.", lablel="popularity_predictor")

        """
        kwargs['collection'] = self.collection
        batch = detect_batch(data)
        data = image_preprocess(data, batch=batch)
        url_params = {"batch": batch, "api_key": api_key, "version": version, 'method': 'remove_example'}
        return api_handler(data, cloud=cloud, api="custom", url_params=url_params, private=True, **kwargs)

    def wait(self, interval=1):
        """
        Block until the collection's model is completed training
        """
        while True:
            status = self.info().get('status')
            if status == "ready":
                break
            if status != "training":
                raise IndicoError("Collection status failed with: {0}".format(status))
            time.sleep(interval)

    def info(self):
        """
        Return the current state of the model associated with a given collection
        """
        return collections().get(self.collection)


def collections(cloud=None, api_key=None, version=None, **kwargs):
    """
    This is a status report endpoint. It is used to get the status on all of the collections currently trained, as
    well as some basic statistics on their accuracies.

    Inputs
    api_key (optional) - String: Your API key, required only if the key has not been declared
      elsewhere. This allows the API to recognize a request as yours and automatically route it
      to the appropriate destination.
    cloud (optional) - String: Your private cloud domain, required only if the key has not been declared
      elsewhere. This allows the API to recognize a request as yours and automatically route it
      to the appropriate destination.

    Example usage:

      .. code-block:: python

         >>> collections = indicoio.collections()
        {
          "tag_predictor": {
            "input_type": "text",
            "model_type": "classification",
            "number_of_samples": 224
            'status': 'ready'
          }, "popularity_predictor": {
            "input_type": "text",
            "model_type": "regression",
            "number_of_samples": 231
            'status': 'training'
          }
        }
      }
    """
    url_params = {"batch": False, "api_key": api_key, "version": version, "method": "collections"}
    return api_handler(None, cloud=cloud, api="custom", url_params=url_params, private=True, **kwargs)



"""
Handles making requests to the IndicoApi Server
"""

import json
import requests
import warnings

try:
    from urllib import urlencode
except: # For Python 3
    from urllib.parse import urlencode

from indicoio.utils.errors import IndicoError
from indicoio import JSON_HEADERS
from indicoio import config

def api_handler(arg, cloud, api, url_params=None, **kwargs):
    """
    Sends finalized request data to ML server and receives response.
    """
    url_params = url_params or {}
    if type(arg) == bytes:
        arg = arg.decode('utf-8')
    if type(arg) == list:
        arg = [a.decode('utf-8') if type(arg) == bytes else a for a in arg]
   
    cloud = cloud or config.cloud
    host = "%s.indico.domains" % cloud if cloud else config.host

    # LOCAL DEPLOYMENTS
    if not (host.endswith('indico.domains') or host.endswith('indico.io')):
        url_protocol = "http"
    else:
        url_protocol = config.url_protocol

    headers = dict(JSON_HEADERS)
    headers["X-ApiKey"] = url_params.get("api_key") or config.api_key
    url = create_url(url_protocol, host, api, dict(kwargs, **url_params))

    data = {'data': arg}
    data.update(**kwargs)
    json_data = json.dumps(data)

    response = requests.post(url, data=json_data, headers=headers)

    warning = response.headers.get('x-warning')
    if warning:
        warnings.warn(warning)

    if response.status_code == 503 and cloud != None:
        raise IndicoError("Private cloud '%s' does not include api '%s'" % (cloud, api))

    json_results = response.json()
    results = json_results.get('results', False)
    if results is False:
        error = json_results.get('error')
        raise IndicoError(error)
    return results


def create_url(url_protocol, host, api, url_params):
    is_batch = url_params.pop("batch", None)
    apis = url_params.pop("apis", None)
    version = url_params.pop("version", None) or url_params.pop("v", None)
    method = url_params.pop('method', None)

    host_url_seg = url_protocol + "://%s" % host
    api_url_seg = "/%s" % api
    batch_url_seg = "/batch" if is_batch else ""
    method_url_seg = "/%s" % method if method else ""

    params = {}
    if apis:
        params["apis"] = ",".join(apis)
    if version:
        params["version"] = version

    url = host_url_seg + api_url_seg + batch_url_seg + method_url_seg
    if params:
        url += "?" + urlencode(params)

    return url



from functools import wraps, partial
import warnings

Version, version, __version__, VERSION = ('0.15.0',) * 4

JSON_HEADERS = {
    'Content-type': 'application/json',
    'Accept': 'application/json',
    'client-lib': 'python',
    'version-number': VERSION
}

from indicoio.text.twitter_engagement import twitter_engagement
from indicoio.text.sentiment import political, posneg, sentiment_hq
from indicoio.text.sentiment import posneg as sentiment
from indicoio.text.lang import language
from indicoio.text.tagging import text_tags
from indicoio.text.keywords import keywords
from indicoio.text.ner import named_entities, people, places, organizations
from indicoio.text.personality import personality
from indicoio.text.personas import personas
from indicoio.text.relevance import relevance
from indicoio.text.text_features import text_features
from indicoio.text.emotion import emotion
from indicoio.images.fer import fer
from indicoio.images.features import facial_features, image_features
from indicoio.images.faciallocalization import facial_localization
from indicoio.images.recognition import image_recognition
from indicoio.images.filtering import content_filtering
from indicoio.utils.multi import analyze_image, analyze_text, intersections
from indicoio.utils.errors import IndicoError



from indicoio.config import TEXT_APIS, IMAGE_APIS, API_NAMES, MULTIAPI_NOT_SUPPORTED
from indicoio.utils.api import api_handler
from indicoio.utils.image import image_preprocess
from indicoio.utils.errors import IndicoError
from indicoio.utils.decorators import detect_batch_decorator


CLIENT_SERVER_MAP = dict((api, api.strip().replace("_", "").lower()) for api in API_NAMES)
SERVER_CLIENT_MAP = dict((v, k) for k, v in CLIENT_SERVER_MAP.items())
AVAILABLE_APIS = {
    'text': TEXT_APIS,
    'image': IMAGE_APIS
}

def invert_dictionary(d):
    return {
        element: key for key, values in d.items()
        for element in values
    }

API_TYPES = invert_dictionary(AVAILABLE_APIS)


@detect_batch_decorator
def intersections(data, apis = None, **kwargs):
    """
    Helper to make multi requests of different types.

    :param data: Data to be sent in API request
    :param type: String type of API request
    :rtype: Dictionary of api responses
    """
    # Client side api name checking
    for api in apis:
        assert api not in MULTIAPI_NOT_SUPPORTED

    # remove auto-inserted batch param
    kwargs.pop('batch', None)

    if not isinstance(apis, list) or len(apis) != 2:
        raise IndicoError("Argument 'apis' must be of length 2")
    if isinstance(data, list) and len(data) < 3:
        raise IndicoError(
            "At least 3 examples are required to use the intersections API"
        )

    api_types = list(map(API_TYPES.get, apis))
    if api_types[0] != api_types[1]:
        raise IndicoError(
            "Both `apis` must accept the same kind of input to use the intersections API"
        )

    cloud = kwargs.pop("cloud", None)

    url_params = {
        'batch': False,
        'api_key': kwargs.pop('api_key', None),
        'apis': apis
    }

    return api_handler(data, cloud=cloud, api="apis/intersections", url_params=url_params, **kwargs)


def multi(data, datatype, apis, batch=False, **kwargs):
    """
    Helper to make multi requests of different types.

    :param data: Data to be sent in API request
    :param datatype: String type of API request
    :param apis: List of apis to use.
    :param batch: Is this a batch request?
    :rtype: Dictionary of api responses
    """
    # Client side api name checking - strictly only accept func name api
    available = list(set(AVAILABLE_APIS.get(datatype)) - set(MULTIAPI_NOT_SUPPORTED))
    invalid_apis = [api for api in apis if api not in available or api in MULTIAPI_NOT_SUPPORTED]
    if invalid_apis:
        raise IndicoError(
            "The following are not valid %s APIs: %s. Please reference the available APIs below:\n%s"
            % (datatype, ", ".join(invalid_apis),  ", ".join(available))
        )

    # Convert client api names to server names before sending request
    cloud = kwargs.pop("cloud", None)
    api_key = kwargs.pop('api_key', None)
    result = api_handler(
        data,
        cloud=cloud,
        api='apis/multiapi',
        url_params={
            "apis":apis,
            "batch":batch,
            "api_key":api_key
        },
        **kwargs
    )
    return handle_response(result)


def handle_response(result):
    # Parse out the results to a dicionary of api: result
    return dict((api, parsed_response(api, res))
        for api, res in result.items())


@detect_batch_decorator
def analyze_text(input_text, apis=None, **kwargs):
    """
    Given input text, returns the results of specified text apis. Possible apis
    include: [ 'text_tags', 'political', 'sentiment', 'language' ]

    Example usage:

    .. code-block:: python

       >>> import indicoio
       >>> text = 'Monday: Delightful with mostly sunny skies. Highs in the low 70s.'
       >>> results = indicoio.analyze_text(data = text, apis = ["language", "sentiment"])
       >>> language_results = results["language"]
       >>> sentiment_results = results["sentiment"]

    :param text: The text to be analyzed.
    :param apis: List of apis to use.
    :type text: str or unicode
    :type apis: list of str
    :rtype: Dictionary of api responses
    """ 

    if not apis:
        apis = list(set(TEXT_APIS) - set(MULTIAPI_NOT_SUPPORTED))

    cloud = kwargs.pop('cloud', None)
    batch = kwargs.pop('batch', False)
    api_key = kwargs.pop('api_key', None)

    return multi(
        data=input_text,
        datatype="text",
        cloud=cloud,
        batch=batch,
        api_key=api_key,
        apis=apis,
        **kwargs
    )


@detect_batch_decorator
def analyze_image(image, apis=IMAGE_APIS, **kwargs):
    """
    Given input image, returns the results of specified image apis. Possible apis
    include: ['fer', 'facial_features', 'image_features']

    Example usage:

    .. code-block:: python

       >>> import indicoio
       >>> import numpy as np
       >>> face = np.zeros((48,48)).tolist()
       >>> results = indicoio.analyze_image(image = face, apis = ["fer", "facial_features"])
       >>> fer = results["fer"]
       >>> facial_features = results["facial_features"]

    :param text: The text to be analyzed.
    :param apis: List of apis to use.
    :type text: str or unicode
    :type apis: list of str
    :rtype: Dictionary of api responses
    """

    if not apis:
        apis = list(set(TEXT_APIS) - set(MULTIAPI_NOT_SUPPORTED))

    cloud = kwargs.pop('cloud', None)
    batch = kwargs.pop('batch', False)
    api_key = kwargs.pop('api_key', None)

    return multi(
        data=image_preprocess(image, batch=batch),
        datatype="image",
        cloud=cloud,
        batch=batch,
        api_key=api_key,
        apis=apis,
        **kwargs
    )

def parsed_response(api, response):
    result = response.get('results', False)
    if result != False:
        return result
    raise IndicoError(
        "Sorry, the %s API returned an unexpected response.\n\t%s"
        % (api, response.get('error', ""))
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-
import unittest
import os, random
from PIL import Image
from requests import ConnectionError

from nose.plugins.skip import Skip, SkipTest
from six import PY3

from indicoio import config
from indicoio import political, sentiment, fer, facial_features, facial_localization, content_filtering, language, image_features, text_tags
from indicoio import keywords, sentiment_hq, twitter_engagement, named_entities, intersections, analyze_image, analyze_text
from indicoio import personas, personality, relevance, people, places, organizations, text_features
from indicoio import emotion
from indicoio.utils.errors import IndicoError

DIR = os.path.dirname(os.path.realpath(__file__))

class BatchAPIRun(unittest.TestCase):

    def setUp(self):
        self.api_key = config.api_key
        config.url_protocol = "http"

        if not all(self.api_key):
            raise SkipTest

    def tearDown(self):
        config.url_protocol = "https"

    def test_batch_texttags(self):
        test_data = ["On Monday, president Barack Obama will be..."]
        response = text_tags(test_data)
        self.assertTrue(isinstance(response, list))

    def test_batch_keywords(self):
        test_data = ["A working api is key to the success of our young company"]
        words = [set(text.lower().split()) for text in test_data]
        response = keywords(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(set(response[0].keys()).issubset(words[0]))

    def test_batch_posneg(self):
        test_data = ['Worst song ever', 'Best song ever']
        response = sentiment(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(response[0] < 0.5)

    def test_batch_sentiment_hq(self):
        test_data = ['Worst song ever', 'Best song ever']
        response = sentiment_hq(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(response[0] < 0.5)

    def test_batch_twitter_engagement(self):
        test_string = "Worst song ever."
        response = twitter_engagement([test_string, test_string])

        self.assertTrue(isinstance(response, list))
        self.assertIsInstance(response[0], float)
        self.assertEqual(response[0], response[1])

    def test_batch_personality(self):
        test_string = "I love my friends!"
        response = personality([test_string,test_string])
        categories = ['extraversion', 'openness', 'agreeableness', 'conscientiousness']
        self.assertTrue(isinstance(response, list))
        self.assertIsInstance(response[0]["extraversion"], float)
        for category in categories:
            assert category in response[0].keys()
        self.assertEqual(response[0]["extraversion"], response[1]["extraversion"])

    def test_batch_personas(self):
        test_string = "I love my friends!"
        response = personas([test_string,test_string])
        self.assertTrue(isinstance(response, list))
        self.assertIsInstance(response[0]["commander"], float)
        self.assertEqual(response[0]["commander"], response[1]["commander"])

    def test_batch_political(self):
        test_data = ["Guns don't kill people, people kill people."]
        response = political(test_data)
        self.assertTrue(isinstance(response, list))

    def test_batch_emotion(self):
        test_data = ["I did it. I got into Grad School. Not just any program, but a GREAT program. :-)"]
        response = emotion(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], dict))
        self.assertIn('joy', response[0].keys())

    def test_url_support(self):
        test_url = "https://s3-us-west-2.amazonaws.com/indico-test-data/face.jpg"
        response = fer(test_url)
        self.assertTrue(isinstance(response, dict))
        self.assertEqual(len(response.keys()), 6)

    def test_batch_fer(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/48by48.png"))]
        response = fer(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], dict))

    def test_batch_content_filtering(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/48by48.png"))]
        response = content_filtering(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], float))

    def test_batch_fer_bad_b64(self):
        test_data = ["$bad#FI jeaf9(#0"]
        self.assertRaises(IndicoError, fer, test_data)

    def test_batch_fer_good_b64(self):
        test_data = ["iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAg5JREFUeNrEV4uNgzAMpegGyAgZgQ3KBscIjMAGx03QEdqbgG5AOwG3AWwAnSCXqLZkuUkwhfYsvaLm5xc7sZ1dIhdtUVjsLZRFTvp+LSaLq8UZ/s+KMSbZCcY5RV9E4QQKHG7QtgeCGv4PFt8WpzkCcztu3TiL0eJgkQmsVFn0MK+LzYkRKEGpG1GDyZdKRdaolhAoJewXnJsO1jtKCFDlChZAFxyJj2PnBRU20KZg7oMlOAENijpi8hwmGkKkZW2GzONtVLA/DxHAhTO2I7MCVBSQ6nGDlEBJDhyVYiUBHXBxzQm0wE4FzPYsGs856dA9SAAP2oENzFYqR6iAFQpHIAUzO/nxnOgthF/lM3w/3U8KYXTwxG/1IgIulF+wPQUXDMl75UoJZIHstRWpaGb8IGYqwBoKlG/lgpzoUEBoj50p8QtVrmHgaaXyC/H3BFC+e9kGFlCB0CtBF7FifQ8D9zjQQHj0pdOM3F1pUBoFKdxtqkMClScHJCSDlSxhHSNRT5K+FaZnHglrz+AGoxZLKNLYH6s3CkkuyJlp58wviZ4PuSCWDXl5hmjZtxcSCGbDUD3gK7EMOZBLCETrgVBF5K0lI5bIZ0wfrYh8NWHIAiNTPHpuTOKpCes1VTFaiNaFdGwPfdmaqlj6LmjJbgoSSfUW74K3voz+/W0oIeB7HWu2s+dfx3N+eLX8CTAAwUmKjK/dHS4AAAAASUVORK5CYII="]
        response = fer(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], dict))

    def test_batch_fer_filepath(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/fear.png"))]
        response = fer(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], dict))

    def test_fer_detect(self):
        test_data = os.path.normpath(os.path.join(DIR, "data/fear.png"))
        response = fer(test_data, detect=True)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 1)
        self.assertIn("location", response[0])

    def test_batch_fer_pil_image(self):
        test_data = [Image.open(os.path.normpath(os.path.join(DIR, "data/fear.png")))]
        response = fer(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], dict))

    def test_batch_fer_nonexistant_filepath(self):
        test_data = ["data/unhappy.png"]
        self.assertRaises(IndicoError, fer, test_data)

    def test_batch_facial_features(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/48by48.png"))]
        response = facial_features(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], list))
        self.assertEqual(len(response[0]), 48)

    def test_batch_image_features_greyscale(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/48by48.png"))]
        response = image_features(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], list))
        self.assertEqual(len(response[0]), 4096)

    def test_batch_image_features_rgb(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/48by48rgb.png"))]
        response = image_features(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(isinstance(response[0], list))
        self.assertEqual(len(response[0]), 4096)

    def test_batch_language(self):
        test_data = ['clearly an english sentence']
        response = language(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(response[0]['English'] > 0.25)

    def test_relevance(self):
        test_data = 'president'
        test_query = ['president', "prime minister"]
        response = relevance(test_data, test_query)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(response[0] > 0.5)
        self.assertTrue(response[1] > 0.3)
        self.assertEqual(len(response), 2)

    def test_batch_relevance(self):
        test_data = ['president', 'president']
        test_query = ['president', "prime minister"]
        response = relevance(test_data, test_query)
        self.assertTrue(isinstance(response, list))
        self.assertTrue(response[0][0] > 0.5)
        self.assertTrue(response[0][1] > 0.3)
        self.assertEqual(len(response), 2)
        self.assertEqual(len(response[0]), 2)
        self.assertEqual(len(response[1]), 2)

    def test_text_features(self):
        test_data = 'Queen of England'
        response = text_features(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 300)

    def test_batch_text_features(self):
        test_data = ['Queen of England', 'Prime Minister of Canada']
        response = text_features(test_data)
        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 2)
        self.assertEqual(len(response[0]), 300)
        self.assertEqual(len(response[1]), 300)

    def test_people(self):
        test_data = 'Barack Obama is scheduled to give a talk next Saturday at the White House.'
        response = people(test_data)
        self.assertTrue(isinstance(response, list))
        sorted_response = sorted(response, key=lambda x: x['confidence'], reverse=True)
        self.assertTrue(sorted_response[0]['text'] == 'Barack Obama')

        test_data = [test_data] * 2
        response = people(test_data)
        self.assertTrue(isinstance(response, list))
        sorted_response = [sorted(arr, key=lambda x: x['confidence'], reverse=True) for arr in response]
        self.assertEqual(len(sorted_response), 2)
        self.assertTrue(sorted_response[0][0]['text'] == 'Barack Obama')

    def test_places(self):
        test_data = "Lets all go to Virginia Beach before it gets too cold to wander outside."
        response = places(test_data)
        self.assertTrue(isinstance(response, list))
        sorted_response = sorted(response, key=lambda x: x['confidence'], reverse=True)
        self.assertTrue('Virginia' in sorted_response[0]['text'])

        test_data = [test_data] * 2
        response = places(test_data)
        self.assertTrue(isinstance(response, list))
        sorted_response = [sorted(arr, key=lambda x: x['confidence'], reverse=True) for arr in response]
        self.assertTrue('Virginia' in sorted_response[0][0]['text'])

    def test_organizations(self):
        test_data = "A year ago, the New York Times published confidential comments about ISIS' ideology by Major General Michael K. Nagata, then U.S. Special Operations commander in the Middle East."
        response = organizations(test_data)
        self.assertTrue(isinstance(response, list))
        sorted_response = sorted(response, key=lambda x: x['confidence'], reverse=True)
        self.assertTrue('ISIS' in sorted_response[0]['text'])

        test_data = [test_data] * 2
        response = organizations(test_data)
        self.assertTrue(isinstance(response, list))
        sorted_response = [sorted(arr, key=lambda x: x['confidence'], reverse=True) for arr in response]
        self.assertTrue('ISIS' in sorted_response[0][0]['text'])

    def test_batch_named_entities(self):
        batch = ["London Underground's boss Mike Brown warned that the strike ..."]
        expected_entities = ("London Underground", "Mike Brown")
        expected_keys = set(["categories", "confidence"])
        entities = named_entities(batch)[0]
        for entity in expected_entities:
            assert entity in expected_entities
            assert not (set(entities[entity]) - expected_keys)

    def test_batch_multi_api_image(self):
        test_data = [os.path.normpath(os.path.join(DIR, "data/48by48.png")),
                     os.path.normpath(os.path.join(DIR, "data/48by48.png"))]
        response = analyze_image(test_data, apis=config.IMAGE_APIS)

        self.assertTrue(isinstance(response, dict))
        self.assertTrue(set(response.keys()) == set(config.IMAGE_APIS))
        self.assertTrue(isinstance(response["fer"], list))

    def test_batch_multi_api_text(self):
        test_data = ['clearly an english sentence']
        response = analyze_text(test_data)

        self.assertTrue(isinstance(response, dict))
        self.assertTrue(set(response.keys()) <= set(config.TEXT_APIS))

    def test_default_multi_api_text(self):
        test_data = ['clearly an english sentence']
        response = analyze_text(test_data)

        self.assertTrue(isinstance(response, dict))
        self.assertTrue(set(response.keys()) <= set(config.TEXT_APIS))

    def test_multi_api_bad_api(self):
        self.assertRaises(IndicoError,
                          analyze_text,
                          "this shouldn't work",
                          apis=["sentiment", "somethingbad"])

    def test_multi_bad_mixed_api(self):
        self.assertRaises(IndicoError,
                            analyze_text,
                            "this shouldn't work",
                            apis=["fer", "sentiment", "facial_features"])

    def test_batch_multi_bad_mixed_api(self):
        self.assertRaises(IndicoError,
                            analyze_text,
                            ["this shouldn't work"],
                            apis=["fer", "sentiment", "facial_features"])

    def test_batch_set_cloud(self):
        test_data = ['clearly an english sentence']
        self.assertRaises(ConnectionError,
                          language,
                          test_data,
                          api_key=self.api_key,
                          cloud='invalid/cloud')


class FullAPIRun(unittest.TestCase):

    def setUp(self):
        self.api_key = config.api_key

    def check_range(self, _list, minimum=0.9, maximum=0.1, span=0.5):
        vector = list(flatten(_list))
        _max = max(vector)
        _min = min(vector)
        self.assertTrue(max(vector) > maximum)
        self.assertTrue(min(vector) < minimum)
        self.assertTrue(_max - _min > span)

    def test_text_tags(self):
        text = "On Monday, president Barack Obama will be..."
        results = text_tags(text)
        max_keys = sorted(results.keys(), key=lambda x:results.get(x), reverse=True)
        assert 'political_discussion' in max_keys[:5]
        results = text_tags(text, top_n=5)
        assert len(results) is 5
        results = text_tags(text, threshold=0.1)
        for v in results.values():
            assert v >= 0.1

    def test_keywords(self):
        text = "A working api is key to the success of our young company"
        words = set(text.lower().split())

        results = keywords(text)
        sorted_results = sorted(results.keys(), key=lambda x:results.get(x), reverse=True)
        assert 'api' in sorted_results[:3]
        self.assertTrue(set(results.keys()).issubset(words))

        results = keywords(text, top_n=3)
        assert len(results) is 3

        results = keywords(text, threshold=.1)
        for v in results.values():
            assert v >= .1

    def test_keywords_language_detect(self):
        text = "il a remporté sa première victoire dans la descente de Val Gardena en Italie"
        words = set(text.lower().split())

        results = keywords(text, language = 'detect')
        sorted_results = sorted(results.keys(), key=lambda x:results.get(x), reverse=True)
        result_keys = results.keys() if PY3 else map(lambda x: x.encode("utf-8"), results.keys())
        self.assertTrue(set(result_keys).issubset(words))

        results = keywords(text, top_n=3)
        assert len(results) is 3

        results = keywords(text, threshold=.1)
        for v in results.values():
            assert v >= .1

    def test_keywords_language(self):
        text = "il a remporté sa première victoire dans la descente de Val Gardena en Italie"
        words = set(text.lower().split())

        results = keywords(text, language = 'French')
        sorted_results = sorted(results.keys(), key=lambda x:results.get(x), reverse=True)

        result_keys = results.keys() if PY3 else map(lambda x: x.encode("utf-8"), results.keys())
        self.assertTrue(set(result_keys).issubset(words))

        results = keywords(text, top_n=3)
        assert len(results) is 3

        results = keywords(text, threshold=.1)
        for v in results.values():
            assert v >= .1

    def test_named_entities(self):
        text = "London Underground's boss Mike Brown warned that the strike ..."
        expected_entities = ("London Underground", "Mike Brown")
        expected_keys = set(["categories", "confidence"])
        entities = named_entities(text)
        for entity in expected_entities:
            assert entity in expected_entities
            assert not (set(entities[entity]) - expected_keys)

    def test_political(self):
        political_set = set(['Libertarian', 'Liberal', 'Conservative', 'Green'])
        test_string = "Guns don't kill people, people kill people."
        response = political(test_string)

        self.assertTrue(isinstance(response, dict))
        self.assertEqual(political_set, set(response.keys()))

        test_string = "pro-choice"
        response = political(test_string)

        self.assertTrue(isinstance(response, dict))
        assert response['Libertarian'] > 0.25

    def test_posneg(self):
        test_string = "Worst song ever."
        response = sentiment(test_string)

        self.assertTrue(isinstance(response, float))
        self.assertTrue(response < 0.5)

        test_string = "Best song ever."
        response = sentiment(test_string)
        self.assertTrue(isinstance(response, float))
        self.assertTrue(response > 0.5)

    def test_sentiment_hq(self):
        test_string = "Worst song ever."
        response = sentiment_hq(test_string)

        self.assertTrue(isinstance(response, float))
        self.assertTrue(response < 0.5)

        test_string = "Best song ever."
        response = sentiment_hq(test_string)
        self.assertTrue(isinstance(response, float))
        self.assertTrue(response > 0.5)

    def test_twitter_engagement(self):
        test_string = "Worst song ever."
        response = twitter_engagement(test_string)

        self.assertIsInstance(response, float)
        self.assertTrue(response <= 1)
        self.assertTrue(response >= 0)

    def test_personalities(self):
        test_string = "I love my friends!"
        response = personality(test_string)

        categories = ['extraversion', 'openness', 'agreeableness', 'conscientiousness']
        self.assertTrue(isinstance(response, dict))
        self.assertIsInstance(response['extraversion'], float)
        for category in categories:
            assert category in response.keys()

    def test_personas(self):
        test_string = "I love my friends!"
        response = personas(test_string)

        self.assertTrue(isinstance(response, dict))
        self.assertIsInstance(response["commander"], float)

    def test_emotion(self):
        data = "I did it. I got into Grad School. Not just any program, but a GREAT program. :-)"
        response = emotion(data)

        self.assertTrue(isinstance(response, dict))
        self.assertIsInstance(response["joy"], float)

    def test_good_fer(self):
        fer_set = set(['Angry', 'Sad', 'Neutral', 'Surprise', 'Fear', 'Happy'])
        test_face = os.path.normpath(os.path.join(DIR, "data/48by48.png"))
        response = fer(test_face)

        self.assertTrue(isinstance(response, dict))
        self.assertEqual(fer_set, set(response.keys()))

    def test_good_int_array_fer(self):
        fer_set = set(['Angry', 'Sad', 'Neutral', 'Surprise', 'Fear', 'Happy'])
        test_face = os.path.normpath(os.path.join(DIR, "data/48by48.png"))
        response = fer(test_face)

        self.assertTrue(isinstance(response, dict))
        self.assertEqual(fer_set, set(response.keys()))

    def test_happy_fer(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/happy.png"))
        response = fer(test_face)
        self.assertTrue(isinstance(response, dict))
        self.assertTrue(response['Happy'] > 0.5)

    def test_happy_fer_pil(self):
        test_face = Image.open(os.path.normpath(os.path.join(DIR, "data/happy.png"))).convert('L');
        response = fer(test_face)
        self.assertTrue(isinstance(response, dict))
        self.assertTrue(response['Happy'] > 0.5)

    def test_fear_fer(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/fear.png"))
        response = fer(test_face)
        self.assertTrue(isinstance(response, dict))
        self.assertTrue(response['Fear'] > 0.25)

    def test_bad_fer(self):
        fer_set = set(['Angry', 'Sad', 'Neutral', 'Surprise', 'Fear', 'Happy'])
        test_face = os.path.normpath(os.path.join(DIR, "data/64by64.png"))
        response = fer(test_face)

        self.assertTrue(isinstance(response, dict))
        self.assertEqual(fer_set, set(response.keys()))

    def test_facial_localization(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/happy.png"))
        res = facial_localization(test_face)[0]
        self.assertTrue(res["top_left_corner"][0] < res["bottom_right_corner"][0])
        self.assertTrue(res["top_left_corner"][1] < res["bottom_right_corner"][1])

    def test_facial_localization_sensitivity(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/happy.png"))
        low_sens = facial_localization(test_face, sensitivity=0.1)
        high_sens = facial_localization(test_face, sensitivity=0.9)
        self.assertEqual(len(low_sens), 1)
        self.assertTrue(len(high_sens) > 1)

    def test_facial_localization_crop(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/happy.png"))
        res = facial_localization(test_face, crop=True)[0]
        self.assertTrue(res.get("image"))

    def test_safe_content_filtering(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/happy.png"))
        response = content_filtering(test_face)
        self.assertTrue(response < 0.5)

    def test_resize_content_filtering(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/happy.png"))
        response = content_filtering(test_face)
        self.assertTrue(isinstance(response, float))

    def test_good_facial_features(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/48by48.png"))
        response = facial_features(test_face)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 48)
        self.check_range(response)

    def test_rgba_int_array_facial_features(self):
        test_face = os.path.normpath(os.path.join(DIR, "data/48by48rgba.png"))
        response = facial_features(test_face)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 48)
        self.check_range(response)

    def test_good_int_array_facial_features(self):
        fer_set = set(['Angry', 'Sad', 'Neutral', 'Surprise', 'Fear', 'Happy'])
        test_face = os.path.normpath(os.path.join(DIR, "data/48by48.png"))
        response = facial_features(test_face)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 48)
        self.check_range(response)

    def test_good_image_features_greyscale(self):
        test_image = os.path.normpath(os.path.join(DIR, "data/48by48.png"))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_good_image_features_rgb(self):
        test_image = os.path.normpath(os.path.join(DIR, "data/48by48rgb.png"))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_multi_api_image(self):
        test_data = os.path.normpath(os.path.join(DIR, "data/48by48.png"))
        response = analyze_image(test_data, apis=config.IMAGE_APIS)

        self.assertTrue(isinstance(response, dict))
        self.assertTrue(set(response.keys()) == set(config.IMAGE_APIS))

    def test_multi_api_text(self):
        test_data = 'clearly an english sentence'
        response = analyze_text(test_data)

        self.assertTrue(isinstance(response, dict))
        self.assertTrue(set(response.keys()) <= set(config.TEXT_APIS))

    def test_intersections_not_enough_data(self):
        test_data = ['test_Data']
        self.assertRaises(
            IndicoError,
            intersections,
            test_data,
            apis=['text_tags', 'sentiment']
        )

    def test_intersections_wrong_number_of_apis(self):
        test_data = ['test data']*3
        self.assertRaises(
            IndicoError,
            intersections,
            test_data,
            apis=['text_tags', 'sentiment', 'language']
        )

    def test_intersections_bad_api_type(self):
        test_data = ['test data']*3
        self.assertRaises(
            IndicoError,
            intersections,
            test_data,
            apis=['text_tags', 'fer']
        )

    def test_intersections_valid_input(self):
        test_data = ['test data']*3
        apis = ['text_tags', 'sentiment']
        results = intersections(test_data, apis=apis)
        assert set(results.keys()) < set(apis)

    def test_intersections_valid_raw_input(self):
        test_data = {
            'sentiment': [0.1, 0.2, 0.3],
            'twitter_engagement': [0.1, 0.2, 0.3]
        }
        results = intersections(test_data, apis=['sentiment', 'twitter_engagement'])
        assert set(results.keys()) < set(test_data.keys())

    def test_language(self):
        language_set = set([
            'English',
            'Spanish',
            'Tagalog',
            'Esperanto',
            'French',
            'Chinese',
            'French',
            'Bulgarian',
            'Latin',
            'Slovak',
            'Hebrew',
            'Russian',
            'German',
            'Japanese',
            'Korean',
            'Portuguese',
            'Italian',
            'Polish',
            'Turkish',
            'Dutch',
            'Arabic',
            'Persian (Farsi)',
            'Czech',
            'Swedish',
            'Indonesian',
            'Vietnamese',
            'Romanian',
            'Greek',
            'Danish',
            'Hungarian',
            'Thai',
            'Finnish',
            'Norwegian',
            'Lithuanian'
        ])
        language_dict = language('clearly an english sentence')
        self.assertEqual(language_set, set(language_dict.keys()))
        assert language_dict['English'] > 0.25

    def test_set_cloud(self):
        test_data = 'clearly an english sentence'
        self.assertRaises(ConnectionError,
                          language,
                          test_data,
                          cloud='invalid/cloud')

        temp_cloud = config.cloud
        config.cloud = 'invalid/cloud'

        self.assertEqual(config.cloud, 'invalid/cloud')
        self.assertRaises(ConnectionError,
                          language,
                          test_data)

        config.cloud = temp_cloud

    def test_set_api_key(self):
        test_data = 'clearly an english sentence'
        self.assertRaises(IndicoError,
                          language,
                          test_data,
                          api_key ='invalid_api_key')

        temp_api_key = config.api_key
        config.api_key = 'invalid_api_key'

        self.assertEqual(config.api_key, 'invalid_api_key')
        self.assertRaises(IndicoError,
                          language,
                          test_data)

        config.api_key = temp_api_key


class NumpyImagesRun(unittest.TestCase):
    """
    Testing numpy array as images
    """
    def setUp(self):
        self.api_key = config.api_key
        try:
            import numpy as np
            globals()["np"] = np
        except ImportError:
            self.skipTest("Numpy is not installed!")

    def check_range(self, _list, minimum=0.9, maximum=0.1, span=0.5):
        vector = list(flatten(_list))
        _max = max(vector)
        _min = min(vector)
        self.assertTrue(max(vector) > maximum)
        self.assertTrue(min(vector) < minimum)
        self.assertTrue(_max - _min > span)

    def test_float_numpy_arrays(self):
        test_image = np.random.random(size=(48,48))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_float_RGB_numpy_arrays(self):
        test_image = np.random.random(size=(48,48,3))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_float_RGBA_numpy_arrays(self):
        test_image = np.random.random(size=(48,48,4))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_int_numpy_arrays(self):
        test_image = np.random.randint(0, 255, size=(48,48))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_int_RGB_numpy_arrays(self):
        test_image = np.random.randint(0, 255, size=(48,48, 3))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_int_RGBA_numpy_arrays(self):
        test_image = np.random.randint(0, 255, size=(48,48, 3))
        response = image_features(test_image)

        self.assertTrue(isinstance(response, list))
        self.assertEqual(len(response), 4096)
        self.check_range(response)

    def test_invalid_int_numpy_arrays(self):
        test_image = np.random.randint(255, 300, size=(48,48, 3))
        self.assertRaises(IndicoError, image_features, test_image)

    def test_invalid_int_numpy_arrays(self):
        test_image = np.random.randint(255, 300, size=(48,48, 5))
        self.assertRaises(IndicoError, image_features, test_image)

    def test_resize_content_filtering_numpy_arrays(self):
        test_image = np.random.randint(0, 255, size=(480,248, 3))
        response = content_filtering(test_image)
        self.assertTrue(isinstance(response, float))

def flatten(container):
    for i in container:
        if isinstance(i, list) or isinstance(i, tuple):
            for j in flatten(i):
                yield j
        else:
            yield i

if __name__ == "__main__":
    unittest.main()



from indicoio.utils.image import image_preprocess
from PIL import Image
import os, unittest, base64
from six import BytesIO

DIR = os.path.dirname(os.path.realpath(__file__))

class ResizeTests(unittest.TestCase):
    """
    test image resizing
    """
    def test_min_axis_resize(self):
        test_image = os.path.normpath(os.path.join(DIR, "data/fear.png"))
        resized_image = image_preprocess(test_image, size=360, min_axis=True)
        image_string = BytesIO(base64.b64decode(resized_image))
        image = Image.open(image_string)
        self.assertEqual(image.size, (360.0, 360.0))
